@@ -34,6 +34,17 @@ Data access
   :py:func:`chain.from_iterable` - read multiple files at once, using an
   iterable of files.
 
+Controlled Vocabularies
+~~~~~~~~~~~~~~~~~~~~~~~
+mzML relies on controlled vocabularies to describe its contents extensibly. See
+`Controlled Vocabulary Terms <../data.html#controlled-vocabulary-terms-in-structured-data>`_
+for more details on how they are used.
+
+Handling Time Units and Other Qualified Quantities
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+mzML contains information which may be described as using a variety of different time units.
+See `Unit Handling <../data.html#unit-handling>`_ for more information.
+
 Deprecated functions
 --------------------
 
@@ -83,10 +94,26 @@ STANDARD_ARRAYS = set([
     'flow rate array',
     'pressure array',
     'temperature array',
-    'mean drift time array',
     'mean charge array',
     'resolution array',
-    'baseline array'
+    'baseline array',
+    'noise array',
+    'sampled noise m/z array',
+    'sampled noise intensity array',
+    'sampled noise baseline array',
+    'ion mobility array',
+    'deconvoluted ion mobility drift time array',
+    'deconvoluted inverse reduced ion mobility array',
+    'deconvoluted ion mobility array',
+    'raw ion mobility drift time array',
+    'raw inverse reduced ion mobility array',
+    'raw ion mobility array',
+    'mean inverse reduced ion mobility array',
+    'mean ion mobility array',
+    'mean ion mobility drift time array',
+    'mass array',
+    'scanning quadrupole position lower bound m/z array',
+    'scanning quadrupole position upper bound m/z array',
 ])
 
 
@@ -141,6 +168,11 @@ class MzML(xml.ArrayConversionMixin, aux.TimeOrderedIndexedReaderMixin, xml.Mult
                     is_non_standard = True
                 else:
                     candidates.append(k)
+        # A non-standard data array term key might have the name for the data array
+        # as the value.
+        nonstandard_name = info.get(NON_STANDARD_DATA_ARRAY)
+        if nonstandard_name:
+            return nonstandard_name
         if isinstance(info.get('name'), list):
             for val in info['name']:
                 if val.endswith(' array'):
@@ -180,9 +212,13 @@ class MzML(xml.ArrayConversionMixin, aux.TimeOrderedIndexedReaderMixin, xml.Mult
         # make a good choice here. We first prefer the standardized
         # arrays before falling back to just guessing.
         else:
+            candidates = set(candidates)
+            # Maybe we just have a repeated term?
+            if len(candidates) == 1:
+                return next(iter(candidates))
             warnings.warn(
                 "Multiple options for naming binary array: %r" % candidates)
-            standard_options = set(candidates) & STANDARD_ARRAYS
+            standard_options = candidates & STANDARD_ARRAYS
             if standard_options:
                 return max(standard_options, key=len)
             return max(candidates, key=len)
