@@ -325,9 +325,13 @@ def DataFrame(*args, **kwargs):
 
     Parameters
     ----------
-    *args, **kwargs : passed to :py:func:`chain`
+    *args
+        Passed to :py:func:`chain`
 
-    sep : str or None, optional
+    **kwargs
+        Passed to :py:func:`chain`
+
+    sep : str or None, keyword only, optional
         Some values related to PSMs (such as protein information) are variable-length
         lists. If `sep` is a :py:class:`str`, they will be packed into single string using
         this delimiter. If `sep` is :py:const:`None`, they are kept as lists. Default is
@@ -395,11 +399,14 @@ def filter_df(*args, **kwargs):
 
     Parameters
     ----------
-    key : str / iterable / callable, optional
-        Default is 'score'. You will probably need to change it.
-    is_decoy : str / iterable / callable, optional
+    key : str / iterable / callable, keyword only, optional
+        Peptide identification score. Default is 'score'. You will probably need to change it.
+    is_decoy : str / iterable / callable, keyword only, optional
         Default is 'is decoy'.
-    *args, **kwargs : passed to :py:func:`auxiliary.filter` and/or :py:func:`DataFrame`.
+    *args
+        Passed to :py:func:`auxiliary.filter` and/or :py:func:`DataFrame`.
+    **kwargs
+        Passed to :py:func:`auxiliary.filter` and/or :py:func:`DataFrame`.
 
     Returns
     -------
